@@ -685,20 +685,6 @@ nd_wfd_p2p_sink_get_peer (NdWFDP2PSink * sink)
   return sink->nm_peer;
 }
 
-/**
- * nd_wfd_p2p_sink_get_state
- * @sink: a #NdWFDP2PSink
- *
- * Retrieve the #NdSinkState.
- *
- * Returns: (transfer none): The #NdSinkState
- */
-NdSinkState
-nd_wfd_p2p_sink_get_state (NdWFDP2PSink *sink)
-{
-  return sink->state;
-}
-
 NdWFDP2PSink *
 nd_wfd_p2p_sink_new (NMClient *client, NMDevice *device, NMWifiP2PPeer * peer)
 {
@@ -727,6 +713,7 @@ nd_wfd_p2p_sink_from_uri (gchar *uri)
   const gchar *protocol_in_uri_str = g_hash_table_lookup (params, "protocol");
 
   NdSinkProtocol protocol_in_uri = g_ascii_strtoll (protocol_in_uri_str, NULL, 10);
+
   if (protocol != protocol_in_uri)
     {
       g_warning ("NdWFDP2PSink: Attempted to create sink whose protocol (%s) doesn't match the URI (%s)",
