@@ -2,41 +2,28 @@
 #include "wfd-media-factory.h"
 #include "wfd-media.h"
 
+static const gchar * wfd_gst_elements[ELEMENT_NONE + 1] = {
+  [ELEMENT_OPENH264] = "openh264enc",
+  [ELEMENT_X264] = "x264enc",
+  [ELEMENT_VAH264] = "vah264enc",
+  [ELEMENT_VAAPIH264] = "vaapih264enc",
+  [ELEMENT_VIDEO_NONE] = NULL,
 
-typedef enum {
-  ENCODER_OPENH264 = 0,
-  ENCODER_X264,
-  ENCODER_VAAPIH264,
-  ENCODER_NONE,
-} WfdH264Encoder;
+  [ELEMENT_AAC_FDK] = "fdkaacenc",
+  [ELEMENT_AAC_AVENC] = "avenc_aac",
+  [ELEMENT_AAC_FAAC] = "faac",
+  [ELEMENT_AUDIO_NONE] = NULL,
 
-static const gchar *h264_encoders[ENCODER_NONE + 1] = {
-  "openh264enc",
-  "x264enc",
-  "vaapih264enc",
-  NULL,
-};
+  [ELEMENT_MPEGTS] = "mpegtsmux",
 
-typedef enum {
-  ENCODER_AAC_FDK = 0,
-  ENCODER_AAC_AVENC,
-  ENCODER_AAC_FAAC,
-  ENCODER_AAC_NONE,
-} WfdAACEncoder;
-
-static const gchar *aac_encoders[ENCODER_AAC_NONE + 1] = {
-  "fdkaacenc",
-  "avenc_aac",
-  "faac",
-  NULL,
+  [ELEMENT_NONE] = NULL,
 };
 
 struct _WfdMediaFactory
 {
   GstRTSPMediaFactory parent_instance;
 
-  WfdH264Encoder      encoder;
-  WfdAACEncoder       aac_encoder;
+  gint                factory_profile;
 };
 
 G_DEFINE_TYPE (WfdMediaFactory, wfd_media_factory, GST_TYPE_RTSP_MEDIA_FACTORY)
@@ -81,7 +68,7 @@ encoding_perf_handoff_cb (GstElement *elem, GstBuffer *buf, gpointer user_data)
       late = MAX (0, now - pts);
 
       /* Ignore the first few frames. */
-      if (pts > 100 * GST_MSECOND)
+      if (pts > 100 * GST_MSECOND && (late - 50 * GST_MSECOND >= 0 || -(late - 50 * GST_MSECOND) <= pts))
         {
           /* We stop accepting things at more than 50ms delay;
            * Just use late / 50ms for the long term proportion. */
@@ -131,18 +118,16 @@ free_qos_data (QOSData *qos_data)
 }
 
 GstElement *
-wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl *url)
+wfd_media_factory_create_video_element (WfdMediaFactory *self, GstBin *bin)
 {
-  g_autoptr(GstBin) bin = NULL;
   g_autoptr(GstCaps) caps = NULL;
-  g_autoptr(GstBin) audio_pipeline = NULL;
   g_autoptr(GstPad) encoding_perf_sink = NULL;
-  WfdMediaFactory *self = WFD_MEDIA_FACTORY (factory);
-  QOSData *qos_data;
   g_autoptr(GstElement) source = NULL;
-  g_autoptr(GstElement) audio_source = NULL;
+
+  QOSData *qos_data;
+
   GstElement *scale;
-  GstElement *sizefilter;
+  GstElement *sinkfilter;
   GstElement *convert;
   GstElement *queue_pre_encoder;
   GstElement *encoder;
@@ -151,12 +136,8 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
   GstElement *parse;
   GstElement *codecfilter;
   GstElement *queue_mpegmux_video;
-  GstElement *mpegmux;
-  GstElement *queue_pre_payloader;
-  GstElement *payloader;
-  gboolean success = TRUE;
 
-  bin = GST_BIN (gst_bin_new ("wfd-encoder-bin"));
+  gboolean success = TRUE;
 
   /* Test input, will be replaced by real source */
   g_signal_emit (self, signals[SIGNAL_CREATE_SOURCE], 0, &source);
@@ -174,9 +155,9 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
                               "width", G_TYPE_INT, 1920,
                               "height", G_TYPE_INT, 1080,
                               NULL);
-  sizefilter = gst_element_factory_make ("capsfilter", "wfd-sizefilter");
-  success &= gst_bin_add (bin, sizefilter);
-  g_object_set (sizefilter,
+  sinkfilter = gst_element_factory_make ("capsfilter", "wfd-sinkfilter");
+  success &= gst_bin_add (bin, sinkfilter);
+  g_object_set (sinkfilter,
                 "caps", caps,
                 NULL);
   g_clear_pointer (&caps, gst_caps_unref);
@@ -194,9 +175,9 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
                 NULL);
   success &= gst_bin_add (bin, queue_pre_encoder);
 
-  switch (self->encoder)
+  switch (wfd_media_factory_profiles[self->factory_profile].video_encoder)
     {
-    case ENCODER_OPENH264:
+    case ELEMENT_OPENH264:
       encoder = gst_element_factory_make ("openh264enc", "wfd-encoder");
       encoder_elem = encoder;
       success &= gst_bin_add (bin, encoder);
@@ -223,7 +204,7 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
        *  - rate-control: 2, buffer*/
       break;
 
-    case ENCODER_X264:
+    case ELEMENT_X264:
       encoder = gst_element_factory_make ("x264enc", "wfd-encoder");
       encoder_elem = encoder;
       success &= gst_bin_add (bin, encoder);
@@ -231,7 +212,21 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
       gst_preset_load_preset (GST_PRESET (encoder), "Profile Baseline");
       break;
 
-    case ENCODER_VAAPIH264:
+    case ELEMENT_VAH264:
+      encoder = gst_element_factory_make ("vah264enc", "wfd-encoder");
+      encoder_elem = encoder;
+      success &= gst_bin_add (bin, encoder);
+      g_object_set (encoder,
+                    "qos", TRUE,
+                    "key-int-max", 30,
+                    "num-slices", 1,
+                    "cabac", FALSE,
+                    "dct8x8", FALSE,
+                    NULL);
+
+      break;
+
+    case ELEMENT_VAAPIH264:
       {
         GstElement *vaapi_encoder;
         GstElement *vaapi_convert;
@@ -239,9 +234,6 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
         encoder = gst_bin_new ("wfd-encoder-bin");
 
         vaapi_convert = gst_element_factory_make ("vaapipostproc", "wfd-vaapi-convert");
-        g_object_set (vaapi_convert,
-                      "format", 2, /* Force I420 */
-                      NULL);
         success &= gst_bin_add (GST_BIN (encoder), vaapi_convert);
 
         vaapi_encoder = gst_element_factory_make ("vaapih264enc", "wfd-encoder");
@@ -279,7 +271,7 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
     default:
       g_assert_not_reached ();
     }
-  g_object_set_data (G_OBJECT (encoder_elem), "wfd-encoder-impl", GINT_TO_POINTER (self->encoder));
+  g_object_set_data (G_OBJECT (encoder_elem), "wfd-encoder-impl", GINT_TO_POINTER (wfd_media_factory_profiles[self->factory_profile].video_encoder));
 
   encoding_perf = gst_element_factory_make ("identity", "wfd-measure-encoder-realtime");
   success &= gst_bin_add (bin, encoding_perf);
@@ -300,7 +292,10 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
                 "config-interval", (gint) - 1,
                 NULL);
 
-  caps = gst_caps_from_string ("video/x-h264,alignment=nal,stream-format=byte-stream,profile=baseline");
+  /* Permit both constrained-baseline and baseline. Would constrained-baseline be sufficient? */
+  caps = gst_caps_from_string ("video/x-h264,stream-format=byte-stream,profile=constrained-baseline");
+  gst_caps_append (caps,
+                   gst_caps_from_string ("video/x-h264,stream-format=byte-stream,profile=baseline"));
   codecfilter = gst_element_factory_make ("capsfilter", "wfd-codecfilter");
   g_object_set (codecfilter,
                 "caps", caps,
@@ -315,6 +310,134 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
                 "max-size-time", 500 * GST_MSECOND,
                 NULL);
 
+  success &= gst_element_link_many (source,
+                                    scale,
+                                    sinkfilter,
+                                    convert,
+                                    queue_pre_encoder,
+                                    encoder,
+                                    encoding_perf,
+                                    parse,
+                                    codecfilter,
+                                    queue_mpegmux_video,
+                                    NULL);
+
+  if (!success)
+    {
+      GST_DEBUG_BIN_TO_DOT_FILE (bin,
+                                 GST_DEBUG_GRAPH_SHOW_MEDIA_TYPE,
+                                 "wfd-video-encoder-bin");
+      g_error ("WfdMediaFactory: Error creating video encoding pipeline. If gstreamer is compiled with debugging and GST_DEBUG_DUMP_DOT_DIR is set, then the pipeline will have been dumped.");
+    }
+
+  return (GstElement *) g_steal_pointer (&queue_mpegmux_video);
+}
+
+GstBin *
+wfd_media_factory_create_audio_element (WfdMediaFactory *self)
+{
+  g_autoptr(GstElement) audio_source = NULL;
+  gboolean success = TRUE;
+
+  if (wfd_media_factory_profiles[self->factory_profile].audio_encoder == ELEMENT_AUDIO_NONE)
+    return NULL;
+
+  g_signal_emit (self, signals[SIGNAL_CREATE_AUDIO_SOURCE], 0, &audio_source);
+
+  if (!audio_source)
+    return NULL;
+
+  g_autoptr(GstCaps) caps = NULL;
+  g_autoptr(GstBin) audio_pipeline = NULL;
+
+  GstElement *audioencoder;
+  GstElement *audioresample;
+  GstElement *audioconvert;
+  GstElement *queue_mpegmux_audio;
+
+  audio_pipeline = GST_BIN (gst_bin_new ("wfd-audio"));
+  /* The audio pipeline is disabled by default, we hook it up and
+   * enable it during configuration. */
+  gst_element_set_locked_state (GST_ELEMENT (audio_pipeline), TRUE);
+
+  success &= gst_bin_add (audio_pipeline, audio_source);
+
+  audioresample = gst_element_factory_make ("audioresample", "wfd-audio-resample");
+  success &= gst_bin_add (audio_pipeline, audioresample);
+
+  audioconvert = gst_element_factory_make ("audioconvert", "wfd-audio-convert");
+  success &= gst_bin_add (audio_pipeline, audioconvert);
+
+  switch (wfd_media_factory_profiles[self->factory_profile].audio_encoder)
+    {
+    case ELEMENT_AAC_FDK:
+      audioencoder = gst_element_factory_make ("fdkaacenc", "wfd-audio-aac-enc");
+      break;
+
+    case ELEMENT_AAC_FAAC:
+      audioencoder = gst_element_factory_make ("faac", "wfd-audio-aac-enc");
+      break;
+
+    case ELEMENT_AAC_AVENC:
+      audioencoder = gst_element_factory_make ("avenc_aac", "wfd-audio-aac-enc");
+      break;
+
+    default:
+      g_assert_not_reached ();
+    }
+  success &= gst_bin_add (audio_pipeline, audioencoder);
+
+  queue_mpegmux_audio = gst_element_factory_make ("queue", "wfd-mpegmux-audio-queue");
+  g_object_set (queue_mpegmux_audio,
+                "max-size-buffers", (guint) 100000,
+                "max-size-time", 500 * GST_MSECOND,
+                "leaky", 0,
+                NULL);
+  success &= gst_bin_add (audio_pipeline, queue_mpegmux_audio);
+
+  caps = gst_caps_new_simple ("audio/mpeg",
+                              "channels", G_TYPE_INT, 2,
+                              "rate", G_TYPE_INT, 48000,
+                              NULL);
+
+  success &= gst_element_link_many (audio_source, audioresample, audioconvert, NULL);
+  success &= gst_element_link (audioconvert, audioencoder);
+  success &= gst_element_link_filtered (audioencoder, queue_mpegmux_audio, caps);
+  g_clear_pointer (&caps, gst_caps_unref);
+
+  gst_element_add_pad (GST_ELEMENT (audio_pipeline),
+                       gst_ghost_pad_new ("src",
+                                          gst_element_get_static_pad (queue_mpegmux_audio,
+                                                                      "src")));
+
+  if (!success)
+    {
+      GST_DEBUG_BIN_TO_DOT_FILE (audio_pipeline,
+                                 GST_DEBUG_GRAPH_SHOW_MEDIA_TYPE,
+                                 "wfd-audio-encoder-bin");
+      g_error ("WfdMediaFactory: Error creating audio encoding pipeline. If gstreamer is compiled with debugging and GST_DEBUG_DUMP_DOT_DIR is set, then the pipeline will have been dumped.");
+    }
+
+  return (GstBin *) g_steal_pointer (&audio_pipeline);
+}
+
+GstElement *
+wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl *url)
+{
+  g_autoptr(GstBin) bin = NULL;
+  g_autoptr(GstBin) audio_pipeline = NULL;
+  WfdMediaFactory *self = WFD_MEDIA_FACTORY (factory);
+
+  GstElement *queue_mpegmux_video;
+  GstElement *mpegmux;
+  GstElement *queue_pre_payloader;
+  GstElement *payloader;
+  gboolean success = TRUE;
+
+  bin = GST_BIN (gst_bin_new ("nd-wfd-pipeline"));
+
+  queue_mpegmux_video = wfd_media_factory_create_video_element (self, bin);
+
   /* TODO: With gstreamer 1.17 we should be able to set things up so
    *       that audio frames are still send even if there are no new
    *       video frames. At that point, some semantics should be
@@ -346,18 +469,6 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
                 "seqnum-offset", (gint) 0,
                 NULL);
 
-  success &= gst_element_link_many (source,
-                                    scale,
-                                    sizefilter,
-                                    convert,
-                                    queue_pre_encoder,
-                                    encoder,
-                                    encoding_perf,
-                                    parse,
-                                    codecfilter,
-                                    queue_mpegmux_video,
-                                    NULL);
-
   /* The WFD specification says we should use stream ID 0x1011. */
   success &= gst_element_link_pads (queue_mpegmux_video, "src", mpegmux, "sink_4113");
   success &= gst_element_link_many (mpegmux,
@@ -367,76 +478,13 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
 
 
   /* Add audio elements */
-  if (self->aac_encoder != ENCODER_AAC_NONE)
-    g_signal_emit (self, signals[SIGNAL_CREATE_AUDIO_SOURCE], 0, &audio_source);
-
-  if (audio_source)
-    {
-      GstElement *audioencoder;
-      GstElement *audioresample;
-      GstElement *audioconvert;
-      GstElement *queue_mpegmux_audio;
-
-      audio_pipeline = GST_BIN (gst_bin_new ("wfd-audio"));
-      success &= gst_bin_add (bin, GST_ELEMENT (g_object_ref (audio_pipeline)));
-      /* The audio pipeline is disabled by default, we hook it up and
-       * enable it during configuration. */
-      gst_element_set_locked_state (GST_ELEMENT (audio_pipeline), TRUE);
-
-      success &= gst_bin_add (audio_pipeline, audio_source);
-
-      audioresample = gst_element_factory_make ("audioresample", "wfd-audio-resample");
-      success &= gst_bin_add (audio_pipeline, audioresample);
-
-      audioconvert = gst_element_factory_make ("audioconvert", "wfd-audio-convert");
-      success &= gst_bin_add (audio_pipeline, audioconvert);
-
-      switch (self->aac_encoder)
-        {
-        case ENCODER_AAC_FDK:
-          audioencoder = gst_element_factory_make ("fdkaacenc", "wfd-audio-aac-enc");
-          break;
-
-        case ENCODER_AAC_FAAC:
-          audioencoder = gst_element_factory_make ("faac", "wfd-audio-aac-enc");
-          break;
-
-        case ENCODER_AAC_AVENC:
-          audioencoder = gst_element_factory_make ("avenc_aac", "wfd-audio-aac-enc");
-          break;
-
-        default:
-          g_assert_not_reached ();
-        }
-      success &= gst_bin_add (audio_pipeline, audioencoder);
-
-      queue_mpegmux_audio = gst_element_factory_make ("queue", "wfd-mpegmux-audio-queue");
-      g_object_set (queue_mpegmux_audio,
-                    "max-size-buffers", (guint) 100000,
-                    "max-size-time", 500 * GST_MSECOND,
-                    "leaky", 0,
-                    NULL);
-      success &= gst_bin_add (audio_pipeline, queue_mpegmux_audio);
-
-      caps = gst_caps_new_simple ("audio/mpeg",
-                                  "channels", G_TYPE_INT, 2,
-                                  "rate", G_TYPE_INT, 48000,
-                                  NULL);
-
-      success &= gst_element_link_many (audio_source, audioresample, audioconvert, NULL);
-      success &= gst_element_link (audioconvert, audioencoder);
-      success &= gst_element_link_filtered (audioencoder, queue_mpegmux_audio, caps);
-      g_clear_pointer (&caps, gst_caps_unref);
-
-      gst_element_add_pad (GST_ELEMENT (audio_pipeline),
-                           gst_ghost_pad_new ("src",
-                                              gst_element_get_static_pad (queue_mpegmux_audio,
-                                                                          "src")));
-    }
+  audio_pipeline = wfd_media_factory_create_audio_element (self);
+  if (audio_pipeline != NULL)
+    success &= gst_bin_add (bin, GST_ELEMENT (g_object_ref (audio_pipeline)));
 
   GST_DEBUG_BIN_TO_DOT_FILE (bin,
                              GST_DEBUG_GRAPH_SHOW_MEDIA_TYPE,
-                             "wfd-encoder-bin");
+                             "nd-wfd-pipeline");
   if (!success)
     {
       g_error ("WfdMediaFactory: Error creating encoding pipeline. If gstreamer is compiled with debugging and GST_DEBUG_DUMP_DOT_DIR is set, then the pipeline will have been dumped.");
@@ -446,74 +494,11 @@ wfd_media_factory_create_element (GstRTSPMediaFactory *factory, const GstRTSPUrl
   return (GstElement *) g_steal_pointer (&bin);
 }
 
-static gboolean
-pipeline_bus_watch_cb (GstBus     *bus,
-                       GstMessage *message,
-                       gpointer    data)
-{
-  switch (GST_MESSAGE_TYPE (message))
-    {
-    case GST_MESSAGE_ERROR: {
-        GError *err;
-        gchar *debug;
-
-        gst_message_parse_error (message, &err, &debug);
-        g_print ("Error from %s: %s\n", GST_MESSAGE_SRC_NAME (message), err->message);
-        g_error_free (err);
-        g_free (debug);
-
-        break;
-      }
-
-    case GST_MESSAGE_WARNING: {
-        GError *err;
-        gchar *debug;
-
-        gst_message_parse_warning (message, &err, &debug);
-        g_print ("Warning from %s: %s\n", GST_MESSAGE_SRC_NAME (message), err->message);
-        g_error_free (err);
-        g_free (debug);
-
-        break;
-      }
-
-    case GST_MESSAGE_QOS: {
-        gboolean live;
-        guint64 running_time, stream_time, timestamp, duration;
-        guint64 processed, dropped;
-        gint64 jitter;
-        gdouble proportion;
-        gint quality;
-
-        gst_message_parse_qos (message, &live, &running_time, &stream_time, &timestamp, &duration);
-        gst_message_parse_qos_stats (message, NULL, &processed, &dropped);
-        gst_message_parse_qos_values (message, &jitter, &proportion, &quality);
-
-        g_debug ("QOS: proportion: %.3f, processed: %" G_GUINT64_FORMAT ", dropped: %" G_GUINT64_FORMAT "",
-                 proportion, processed, dropped);
-        break;
-      }
-
-    case GST_MESSAGE_EOS:
-      /* end-of-stream */
-      g_debug ("reached EOS");
-      break;
-
-    default:
-      /* unhandled message */
-      break;
-    }
-
-  return TRUE;
-}
-
 static GstElement *
 wfd_media_factory_create_pipeline (GstRTSPMediaFactory *factory, GstRTSPMedia *media)
 {
   GstElement *pipeline;
 
-  g_autoptr(GstBus) bus = NULL;
-
   pipeline = GST_RTSP_MEDIA_FACTORY_CLASS (wfd_media_factory_parent_class)->create_pipeline (factory, media);
 
   /* We need a high latency for the openh264 encoder at least when the
@@ -521,18 +506,15 @@ wfd_media_factory_create_pipeline (GstRTSPMediaFactory *factory, GstRTSPMedia *m
    * be very high for short periods of time, and this prevents further issues. */
   gst_pipeline_set_latency (GST_PIPELINE (pipeline), 500 * GST_MSECOND);
 
-  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
-  gst_bus_add_watch (bus, pipeline_bus_watch_cb, NULL);
-
   return pipeline;
 }
 
 WfdMediaQuirks
 wfd_configure_media_element (GstBin *bin, WfdParams *params)
 {
-  g_autoptr(GstCaps) caps_sizefilter = NULL;
-  g_autoptr(GstElement) sizefilter = NULL;
-  g_autoptr(GstCaps) caps_codecfilter = NULL;
+  g_autoptr(GstCaps) caps = NULL;
+  g_autoptr(GstElement) srcfilter = NULL;
+  g_autoptr(GstElement) sinkfilter = NULL;
   g_autoptr(GstElement) codecfilter = NULL;
   g_autoptr(GstElement) encoder = NULL;
   g_autoptr(GstElement) audio_pipeline = NULL;
@@ -540,7 +522,7 @@ wfd_configure_media_element (GstBin *bin, WfdParams *params)
   WfdMediaQuirks quirks = 0;
   WfdVideoCodec *codec = params->selected_codec;
   WfdResolution *resolution = params->selected_resolution;
-  WfdH264Encoder encoder_impl;
+  WfdGstElement encoder_impl;
   WfdH264ProfileFlags profile;
   guint gop_size = resolution->refresh_rate;
   guint bitrate_kbit = wfd_video_codec_get_max_bitrate_kbit (codec);
@@ -556,7 +538,7 @@ wfd_configure_media_element (GstBin *bin, WfdParams *params)
   encoder = gst_bin_get_by_name (bin, "wfd-encoder");
   encoder_impl = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (encoder), "wfd-encoder-impl"));
 
-  if (encoder_impl == ENCODER_VAAPIH264)
+  if (encoder_impl == ELEMENT_VAAPIH264)
     quirks = WFD_QUIRK_NO_IDR;
 
   /* Decrease the number of keyframes if the device is able to request
@@ -565,20 +547,36 @@ wfd_configure_media_element (GstBin *bin, WfdParams *params)
   if (params->idr_request_capability && !(quirks & WFD_QUIRK_NO_IDR))
     gop_size = 10 * resolution->refresh_rate;
 
-  caps_sizefilter = gst_caps_new_simple ("video/x-raw",
-                                         "framerate", GST_TYPE_FRACTION, resolution->refresh_rate, 1,
-                                         "width", G_TYPE_INT, resolution->width,
-                                         "height", G_TYPE_INT, resolution->height,
-                                         NULL);
+  srcfilter = gst_bin_get_by_name (bin, "srcfilter");
+  if (srcfilter != NULL)
+    {
+      caps = gst_caps_new_simple ("video/x-raw",
+                                  "max-framerate", GST_TYPE_FRACTION, resolution->refresh_rate, 1,
+                                  "width", G_TYPE_INT, resolution->width,
+                                  "height", G_TYPE_INT, resolution->height,
+                                  NULL);
 
-  sizefilter = gst_bin_get_by_name (bin, "wfd-sizefilter");
-  g_object_set (sizefilter,
-                "caps", caps_sizefilter,
+      g_object_set (srcfilter,
+                    "caps", caps,
+                    NULL);
+      g_clear_pointer (&caps, gst_caps_unref);
+    }
+
+  caps = gst_caps_new_simple ("video/x-raw",
+                              "framerate", GST_TYPE_FRACTION, resolution->refresh_rate, 1,
+                              "width", G_TYPE_INT, resolution->width,
+                              "height", G_TYPE_INT, resolution->height,
+                              NULL);
+
+  sinkfilter = gst_bin_get_by_name (bin, "wfd-sinkfilter");
+  g_object_set (sinkfilter,
+                "caps", caps,
                 NULL);
+  g_clear_pointer (&caps, gst_caps_unref);
 
   switch (encoder_impl)
     {
-    case ENCODER_OPENH264:
+    case ELEMENT_OPENH264:
       /* We could set multi-thread/num-slices to codec->max_slice_num; but not sure
        * if that works realiably, and simply using one slice is on the safe side
        */
@@ -590,10 +588,10 @@ wfd_configure_media_element (GstBin *bin, WfdParams *params)
                     NULL);
       break;
 
-    case ENCODER_X264:
+    case ELEMENT_X264:
       if (codec->profile == WFD_H264_PROFILE_HIGH)
         {
-          profile = WFD_H264_PROFILE_HIGH;
+          profile = codec->profile;
           gst_preset_load_preset (GST_PRESET (encoder), "Profile High");
         }
       else
@@ -625,8 +623,19 @@ wfd_configure_media_element (GstBin *bin, WfdParams *params)
                     NULL);
       break;
 
+    case ELEMENT_VAH264:
+      if (codec->profile == WFD_H264_PROFILE_HIGH)
+        profile = WFD_H264_PROFILE_HIGH;
+      else
+        profile = WFD_H264_PROFILE_BASE;
 
-    case ENCODER_VAAPIH264:
+      g_object_set (encoder,
+                    "key-int-max", (guint) gop_size,
+                    "bitrate",  bitrate_kbit,
+                    NULL);
+      break;
+
+    case ELEMENT_VAAPIH264:
       if (codec->profile == WFD_H264_PROFILE_HIGH)
         profile = WFD_H264_PROFILE_HIGH;
       else
@@ -643,15 +652,20 @@ wfd_configure_media_element (GstBin *bin, WfdParams *params)
     }
 
   if (profile == WFD_H264_PROFILE_HIGH)
-    caps_codecfilter = gst_caps_from_string ("video/x-h264,alignment=nal,stream-format=byte-stream,profile=high");
+    caps = gst_caps_from_string ("video/x-h264,stream-format=byte-stream,profile=high");
   else
-    caps_codecfilter = gst_caps_from_string ("video/x-h264,alignment=nal,stream-format=byte-stream,profile=baseline");
+    {
+      /* Permit both constrained-baseline and baseline. Would constrained-baseline be sufficient? */
+      caps = gst_caps_from_string ("video/x-h264,stream-format=byte-stream,profile=constrained-baseline");
+      gst_caps_append (caps,
+                       gst_caps_from_string ("video/x-h264,stream-format=byte-stream,profile=baseline"));
+    }
 
   codecfilter = gst_bin_get_by_name (bin, "wfd-codecfilter");
   g_object_set (codecfilter,
-                "caps", caps_codecfilter,
+                "caps", caps,
                 NULL);
-
+  g_clear_pointer (&caps, gst_caps_unref);
 
   g_debug ("An audiocodec has been selected: %s", params->selected_audio_codec ? "yes" : "no");
   audio_pipeline = gst_bin_get_by_name (bin, "wfd-audio");
@@ -680,7 +694,7 @@ wfd_configure_media_element (GstBin *bin, WfdParams *params)
 
   GST_DEBUG_BIN_TO_DOT_FILE (bin,
                              GST_DEBUG_GRAPH_SHOW_ALL,
-                             "wfd-encoder-bin-configured");
+                             "nd-wfd-pipeline-configured");
 
   return quirks;
 }
@@ -706,7 +720,7 @@ wfd_media_factory_new (void)
   return g_object_new (WFD_TYPE_MEDIA_FACTORY, NULL);
 }
 
-static void
+void
 wfd_media_factory_finalize (GObject *object)
 {
   g_debug ("WfdMediaFactory: Finalize");
@@ -742,74 +756,158 @@ wfd_media_factory_class_init (WfdMediaFactoryClass *klass)
 }
 
 static gboolean
+wfd_gst_element_present (WfdGstElement element)
+{
+  g_autoptr(GstElementFactory) encoder_factory = NULL;
+
+  /* -1 = not found
+   *  0 = not checked
+   *  1 = found
+   */
+  static gint8 found_elements[ELEMENT_NONE] = {0};
+
+  if (element == ELEMENT_VIDEO_NONE || element == ELEMENT_AUDIO_NONE)
+    return TRUE; /* is "nothing" found? yes! */
+
+  if (found_elements[element] != 0)
+    return found_elements[element] == 1;
+
+  encoder_factory = gst_element_factory_find (wfd_gst_elements[element]);
+
+  if (encoder_factory != NULL)
+    {
+      g_debug ("WfdMediaFactory: Found %s gst element.", wfd_gst_elements[element]);
+      found_elements[element] = 1;
+      return TRUE;
+    }
+
+  found_elements[element] = -1;
+  return FALSE;
+}
+
+static gint
+wfd_gst_media_profile_present (WfdMediaProfile media_profile)
+{
+  gint factory_profile;
+
+  g_debug ("WfdMediaFactory: Checking profile: %d", media_profile);
+
+  if (media_profile < 0 || media_profile >= PROFILE_LAST)
+    return -1;
+
+  for (factory_profile = 0; factory_profile < (sizeof (wfd_media_factory_profiles) / sizeof (wfd_media_factory_profiles[0])); ++factory_profile)
+    {
+      if (wfd_media_factory_profiles[factory_profile].media_profile == media_profile &&
+          wfd_gst_element_present (wfd_media_factory_profiles[factory_profile].video_encoder) &&
+          wfd_gst_element_present (wfd_media_factory_profiles[factory_profile].audio_encoder) &&
+          wfd_gst_element_present (wfd_media_factory_profiles[factory_profile].muxer))
+        {
+          g_debug ("WfdMediaFactory: Found elements for media profile: %d", media_profile);
+          return factory_profile;
+        }
+    }
+
+  return -1;
+}
+
+gboolean
 wfd_media_factory_lookup_encoders (WfdMediaFactory *self,
+                                   WfdMediaProfile  media_profile,
                                    GStrv           *missing_video,
                                    GStrv           *missing_audio)
 {
-  WfdH264Encoder h264_encoder, h264_selected;
-  WfdAACEncoder aac_encoder, aac_selected;
+  gint factory_profile;
 
-  /* Default to openh264 and assume it is usable, prefer x264enc when available. */
-  h264_selected = ENCODER_NONE;
+  *missing_video = NULL;
+  *missing_audio = NULL;
 
-  for (h264_encoder = ENCODER_OPENH264; h264_encoder < ENCODER_NONE; h264_encoder++)
+  /* check for particular profile */
+  if (media_profile != PROFILE_LAST)
     {
-      g_autoptr(GstElementFactory) encoder_factory = NULL;
+      factory_profile = wfd_gst_media_profile_present (media_profile);
+      if (factory_profile == -1)
+        goto missing_elements;
 
-      encoder_factory = gst_element_factory_find (h264_encoders[h264_encoder]);
-      if (encoder_factory)
+      self->factory_profile = factory_profile;
+      return TRUE;
+    }
+
+  /* PROFILE_HIGH_H264 */
+  if ((factory_profile = wfd_gst_media_profile_present (PROFILE_HIGH_H264)) != -1)
+    {
+      self->factory_profile = factory_profile;
+      return TRUE;
+    }
+
+  /* PROFILE_BASE_H264 */
+  if ((factory_profile = wfd_gst_media_profile_present (PROFILE_BASE_H264)) != -1)
+    {
+      self->factory_profile = factory_profile;
+      return TRUE;
+    }
+
+missing_elements:
+  switch (media_profile)
+    {
+    case PROFILE_LAST:
+    case PROFILE_BASE_H264:
+      if  (!wfd_gst_element_present (ELEMENT_OPENH264) &&
+           !wfd_gst_element_present (ELEMENT_X264) &&
+           !wfd_gst_element_present (ELEMENT_VAH264) &&
+           !wfd_gst_element_present (ELEMENT_VAAPIH264))
         {
-          g_debug ("Found %s for video encoding.", h264_encoders[h264_encoder]);
-          h264_selected = h264_encoder;
-
-          /* Don't continue searching if the user specified this encoder. */
-          if (g_strcmp0 (g_getenv ("NETWORK_DISPLAYS_H264_ENC"), h264_encoders[h264_encoder]) == 0)
-            break;
+          gchar *missing[5] = { (gchar *) wfd_gst_elements[ELEMENT_OPENH264],
+                                (gchar *) wfd_gst_elements[ELEMENT_X264],
+                                (gchar *) wfd_gst_elements[ELEMENT_VAH264],
+                                (gchar *) wfd_gst_elements[ELEMENT_VAAPIH264],
+                                NULL };
+          *missing_video = (GStrv) g_strdupv (missing);
         }
-    }
-
-  if (h264_selected == ENCODER_NONE)
-    {
-      g_debug ("WFD: Did not find any usable H264 video encoder, missing dependencies!");
-
-      if (missing_video)
-        *missing_video = (GStrv) h264_encoders;
-    }
-
-  aac_selected = ENCODER_AAC_NONE;
-
-  for (aac_encoder = ENCODER_AAC_FDK; aac_encoder < ENCODER_AAC_NONE; aac_encoder++)
-    {
-      g_autoptr(GstElementFactory) encoder_factory = NULL;
-
-      encoder_factory = gst_element_factory_find (aac_encoders[aac_encoder]);
-      if (encoder_factory)
+      /* use missing_video for muxer as well (legacy reasons) */
+      else if (!wfd_gst_element_present (ELEMENT_MPEGTS))
         {
-          g_debug ("Found %s for audio encoding.", aac_encoders[aac_encoder]);
-          aac_selected = aac_encoder;
-
-          /* Don't continue searching if the user specified this encoder. */
-          if (g_strcmp0 (g_getenv ("NETWORK_DISPLAYS_AAC_ENC"), aac_encoders[aac_encoder]) == 0)
-            break;
+          gchar *missing[2] = { (gchar *) wfd_gst_elements[ELEMENT_MPEGTS], NULL };
+          *missing_video = (GStrv) g_strdupv (missing);
         }
+
+      break;
+
+    case PROFILE_HIGH_H264:
+      if (!wfd_gst_element_present (ELEMENT_X264) &&
+          !wfd_gst_element_present (ELEMENT_VAH264) &&
+          !wfd_gst_element_present (ELEMENT_VAAPIH264))
+        {
+          gchar *missing[4] = { (gchar *) wfd_gst_elements[ELEMENT_X264],
+                                (gchar *) wfd_gst_elements[ELEMENT_VAH264],
+                                (gchar *) wfd_gst_elements[ELEMENT_VAAPIH264],
+                                NULL };
+          *missing_video = (GStrv) g_strdupv (missing);
+        }
+      /* use missing_video for muxer as well (legacy reasons) */
+      else if (!wfd_gst_element_present (ELEMENT_MPEGTS))
+        {
+          gchar *missing[2] = { (gchar *) wfd_gst_elements[ELEMENT_MPEGTS], NULL };
+          *missing_video = (GStrv) g_strdupv (missing);
+        }
+
+      break;
+
+    default:
+      break;
     }
 
-  if (aac_selected == ENCODER_AAC_NONE)
+  if (!wfd_gst_element_present (ELEMENT_AAC_FDK) &&
+      !wfd_gst_element_present (ELEMENT_AAC_AVENC) &&
+      !wfd_gst_element_present (ELEMENT_AAC_FAAC))
     {
-      g_debug ("WFD: Did not find any usable AAC audio encoder!");
-
-      if (missing_audio)
-        *missing_audio = (GStrv) aac_encoders;
+      gchar *missing[4] = { (gchar *) wfd_gst_elements[ELEMENT_AAC_FDK],
+                            (gchar *) wfd_gst_elements[ELEMENT_AAC_AVENC],
+                            (gchar *) wfd_gst_elements[ELEMENT_AAC_FAAC],
+                            NULL };
+      *missing_audio = (GStrv) g_strdupv (missing);
     }
 
-
-  if (self)
-    {
-      self->encoder = h264_selected;
-      self->aac_encoder = aac_selected;
-    }
-
-  return h264_selected != ENCODER_NONE;
+  return FALSE;
 }
 
 static void
@@ -817,18 +915,7 @@ wfd_media_factory_init (WfdMediaFactory *self)
 {
   GstRTSPMediaFactory *media_factory = GST_RTSP_MEDIA_FACTORY (self);
 
-  g_assert (wfd_media_factory_lookup_encoders (self, NULL, NULL));
-
   gst_rtsp_media_factory_set_media_gtype (media_factory, WFD_TYPE_MEDIA);
   gst_rtsp_media_factory_set_suspend_mode (media_factory, GST_RTSP_SUSPEND_MODE_RESET);
   gst_rtsp_media_factory_set_buffer_size (media_factory, 65536);
 }
-
-gboolean
-wfd_get_missing_codecs (GStrv *video, GStrv *audio)
-{
-  *video = NULL;
-  *audio = NULL;
-
-  return wfd_media_factory_lookup_encoders (NULL, video, audio);
-}
